# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .shift_wage import ShiftWage
from .break_ import Break
from .shift_status import ShiftStatus
from .money import Money
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Shift(UncheckedBaseModel):
    """
    A record of the hourly rate, start, and end times for a single work shift
    for an employee. This might include a record of the start and end times for breaks
    taken during the shift.

    Deprecated at Square API version 2025-05-21. Replaced by [Timecard](entity:Timecard).
    See the [migration notes](https://developer.squareup.com/docs/labor-api/what-it-does#migration-notes).
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The UUID for this object.
    """

    employee_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the employee this shift belongs to. DEPRECATED at version 2020-08-26. Use `team_member_id` instead.
    """

    location_id: str = pydantic.Field()
    """
    The ID of the location this shift occurred at. The location should be based on
    where the employee clocked in.
    """

    timezone: typing.Optional[str] = pydantic.Field(default=None)
    """
    The read-only convenience value that is calculated from the location based
    on the `location_id`. Format: the IANA timezone database identifier for the
    location timezone.
    """

    start_at: str = pydantic.Field()
    """
    RFC 3339; shifted to the location timezone + offset. Precision up to the
    minute is respected; seconds are truncated.
    """

    end_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    RFC 3339; shifted to the timezone + offset. Precision up to the minute is
    respected; seconds are truncated.
    """

    wage: typing.Optional[ShiftWage] = pydantic.Field(default=None)
    """
    Job and pay related information. If the wage is not set on create, it defaults to a wage
    of zero. If the title is not set on create, it defaults to the name of the role the employee
    is assigned to, if any.
    """

    breaks: typing.Optional[typing.List[Break]] = pydantic.Field(default=None)
    """
    A list of all the paid or unpaid breaks that were taken during this shift.
    """

    status: typing.Optional[ShiftStatus] = pydantic.Field(default=None)
    """
    Describes the working state of the current `Shift`.
    See [ShiftStatus](#type-shiftstatus) for possible values
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    Used for resolving concurrency issues. The request fails if the version
    provided does not match the server version at the time of the request. If not provided,
    Square executes a blind write; potentially overwriting data from another
    write.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A read-only timestamp in RFC 3339 format; presented in UTC.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A read-only timestamp in RFC 3339 format; presented in UTC.
    """

    team_member_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the team member this shift belongs to. Replaced `employee_id` at version "2020-08-26".
    """

    declared_cash_tip_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The tips declared by the team member for the shift.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.inventory_state import InventoryState


class InventoryCountParams(typing_extensions.TypedDict):
    """
    Represents Square-estimated quantity of items in a particular state at a
    particular seller location based on the known history of physical counts and
    inventory adjustments.
    """

    catalog_object_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-generated ID of the
    [CatalogObject](entity:CatalogObject) being tracked.
    """

    catalog_object_type: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The [type](entity:CatalogObjectType) of the [CatalogObject](entity:CatalogObject) being tracked. 
    
    The Inventory API supports setting and reading the `"catalog_object_type": "ITEM_VARIATION"` field value. 
    In addition, it can also read the `"catalog_object_type": "ITEM"` field value that is set by the Square Restaurants app.
    """

    state: typing_extensions.NotRequired[InventoryState]
    """
    The current [inventory state](entity:InventoryState) for the related
    quantity of items.
    See [InventoryState](#type-inventorystate) for possible values
    """

    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The Square-generated ID of the [Location](entity:Location) where the related
    quantity of items is being tracked.
    """

    quantity: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The number of items affected by the estimated count as a decimal string.
    Can support up to 5 digits after the decimal point.
    """

    calculated_at: typing_extensions.NotRequired[str]
    """
    An RFC 3339-formatted timestamp that indicates when the most recent physical count or adjustment affecting
    the estimated count is received.
    """

    is_estimated: typing_extensions.NotRequired[bool]
    """
    Whether the inventory count is for composed variation (TRUE) or not (FALSE). If true, the inventory count will not be present in the response of
    any of these endpoints: [BatchChangeInventory](api-endpoint:Inventory-BatchChangeInventory),
    [BatchRetrieveInventoryChanges](api-endpoint:Inventory-BatchRetrieveInventoryChanges),
    [BatchRetrieveInventoryCounts](api-endpoint:Inventory-BatchRetrieveInventoryCounts), and
    [RetrieveInventoryChanges](api-endpoint:Inventory-RetrieveInventoryChanges).
    """



# This file was auto-generated by Fern from our API Definition.

import typing

TransactionProduct = typing.Union[
    typing.Literal[
        "REGISTER", "EXTERNAL_API", "BILLING", "APPOINTMENTS", "INVOICES", "ONLINE_STORE", "PAYROLL", "OTHER"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .confirmation_decision import ConfirmationDecision
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ConfirmationOptions(UncheckedBaseModel):
    title: str = pydantic.Field()
    """
    The title text to display in the confirmation screen flow on the Terminal.
    """

    body: str = pydantic.Field()
    """
    The agreement details to display in the confirmation flow on the Terminal.
    """

    agree_button_text: str = pydantic.Field()
    """
    The button text to display indicating the customer agrees to the displayed terms.
    """

    disagree_button_text: typing.Optional[str] = pydantic.Field(default=None)
    """
    The button text to display indicating the customer does not agree to the displayed terms.
    """

    decision: typing.Optional[ConfirmationDecision] = pydantic.Field(default=None)
    """
    The result of the buyer’s actions when presented with the confirmation screen.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ....core.client_wrapper import SyncClientWrapper
from .raw_client import RawPromotionsClient
from ....types.loyalty_promotion_status import LoyaltyPromotionStatus
from ....core.request_options import RequestOptions
from ....core.pagination import SyncPager
from ....types.loyalty_promotion import LoyaltyPromotion
from ....core.jsonable_encoder import jsonable_encoder
from ....types.list_loyalty_promotions_response import ListLoyaltyPromotionsResponse
from ....core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ....core.api_error import ApiError
from ....requests.loyalty_promotion import LoyaltyPromotionParams
from ....types.create_loyalty_promotion_response import CreateLoyaltyPromotionResponse
from ....types.get_loyalty_promotion_response import GetLoyaltyPromotionResponse
from ....types.cancel_loyalty_promotion_response import CancelLoyaltyPromotionResponse
from ....core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawPromotionsClient
from ....core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PromotionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPromotionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPromotionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPromotionsClient
        """
        return self._raw_client

    def list(
        self,
        program_id: str,
        *,
        status: typing.Optional[LoyaltyPromotionStatus] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[LoyaltyPromotion]:
        """
        Lists the loyalty promotions associated with a [loyalty program](entity:LoyaltyProgram).
        Results are sorted by the `created_at` date in descending order (newest to oldest).

        Parameters
        ----------
        program_id : str
            The ID of the base [loyalty program](entity:LoyaltyProgram). To get the program ID,
            call [RetrieveLoyaltyProgram](api-endpoint:Loyalty-RetrieveLoyaltyProgram) using the `main` keyword.

        status : typing.Optional[LoyaltyPromotionStatus]
            The status to filter the results by. If a status is provided, only loyalty promotions
            with the specified status are returned. Otherwise, all loyalty promotions associated with
            the loyalty program are returned.

        cursor : typing.Optional[str]
            The cursor returned in the paged response from the previous call to this endpoint.
            Provide this cursor to retrieve the next page of results for your original request.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The maximum number of results to return in a single paged response.
            The minimum value is 1 and the maximum value is 30. The default value is 30.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[LoyaltyPromotion]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.loyalty.programs.promotions.list(
            program_id="program_id",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            f"v2/loyalty/programs/{jsonable_encoder(program_id)}/promotions",
            method="GET",
            params={
                "status": status,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListLoyaltyPromotionsResponse,
                    construct_type(
                        type_=ListLoyaltyPromotionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    program_id,
                    status=status,
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.loyalty_promotions
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        program_id: str,
        *,
        loyalty_promotion: LoyaltyPromotionParams,
        idempotency_key: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateLoyaltyPromotionResponse:
        """
        Creates a loyalty promotion for a [loyalty program](entity:LoyaltyProgram). A loyalty promotion
        enables buyers to earn points in addition to those earned from the base loyalty program.

        This endpoint sets the loyalty promotion to the `ACTIVE` or `SCHEDULED` status, depending on the
        `available_time` setting. A loyalty program can have a maximum of 10 loyalty promotions with an
        `ACTIVE` or `SCHEDULED` status.

        Parameters
        ----------
        program_id : str
            The ID of the [loyalty program](entity:LoyaltyProgram) to associate with the promotion.
            To get the program ID, call [RetrieveLoyaltyProgram](api-endpoint:Loyalty-RetrieveLoyaltyProgram)
            using the `main` keyword.

        loyalty_promotion : LoyaltyPromotionParams
            The loyalty promotion to create.

        idempotency_key : str
            A unique identifier for this request, which is used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateLoyaltyPromotionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.programs.promotions.create(
            program_id="program_id",
            loyalty_promotion={
                "name": "Tuesday Happy Hour Promo",
                "incentive": {
                    "type": "POINTS_MULTIPLIER",
                    "points_multiplier_data": {"multiplier": "3.0"},
                },
                "available_time": {
                    "time_periods": [
                        "BEGIN:VEVENT\nDTSTART:20220816T160000\nDURATION:PT2H\nRRULE:FREQ=WEEKLY;BYDAY=TU\nEND:VEVENT"
                    ]
                },
                "trigger_limit": {"times": 1, "interval": "DAY"},
                "minimum_spend_amount_money": {"amount": 2000, "currency": "USD"},
                "qualifying_category_ids": ["XTQPYLR3IIU9C44VRCB3XD12"],
            },
            idempotency_key="ec78c477-b1c3-4899-a209-a4e71337c996",
        )
        """
        response = self._raw_client.create(
            program_id,
            loyalty_promotion=loyalty_promotion,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    def get(
        self, promotion_id: str, program_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetLoyaltyPromotionResponse:
        """
        Retrieves a loyalty promotion.

        Parameters
        ----------
        promotion_id : str
            The ID of the [loyalty promotion](entity:LoyaltyPromotion) to retrieve.

        program_id : str
            The ID of the base [loyalty program](entity:LoyaltyProgram). To get the program ID,
            call [RetrieveLoyaltyProgram](api-endpoint:Loyalty-RetrieveLoyaltyProgram) using the `main` keyword.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetLoyaltyPromotionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.programs.promotions.get(
            promotion_id="promotion_id",
            program_id="program_id",
        )
        """
        response = self._raw_client.get(promotion_id, program_id, request_options=request_options)
        return response.data

    def cancel(
        self, promotion_id: str, program_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CancelLoyaltyPromotionResponse:
        """
        Cancels a loyalty promotion. Use this endpoint to cancel an `ACTIVE` promotion earlier than the
        end date, cancel an `ACTIVE` promotion when an end date is not specified, or cancel a `SCHEDULED` promotion.
        Because updating a promotion is not supported, you can also use this endpoint to cancel a promotion before
        you create a new one.

        This endpoint sets the loyalty promotion to the `CANCELED` state

        Parameters
        ----------
        promotion_id : str
            The ID of the [loyalty promotion](entity:LoyaltyPromotion) to cancel. You can cancel a
            promotion that has an `ACTIVE` or `SCHEDULED` status.

        program_id : str
            The ID of the base [loyalty program](entity:LoyaltyProgram).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CancelLoyaltyPromotionResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.programs.promotions.cancel(
            promotion_id="promotion_id",
            program_id="program_id",
        )
        """
        response = self._raw_client.cancel(promotion_id, program_id, request_options=request_options)
        return response.data


class AsyncPromotionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPromotionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPromotionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPromotionsClient
        """
        return self._raw_client

    async def list(
        self,
        program_id: str,
        *,
        status: typing.Optional[LoyaltyPromotionStatus] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[LoyaltyPromotion]:
        """
        Lists the loyalty promotions associated with a [loyalty program](entity:LoyaltyProgram).
        Results are sorted by the `created_at` date in descending order (newest to oldest).

        Parameters
        ----------
        program_id : str
            The ID of the base [loyalty program](entity:LoyaltyProgram). To get the program ID,
            call [RetrieveLoyaltyProgram](api-endpoint:Loyalty-RetrieveLoyaltyProgram) using the `main` keyword.

        status : typing.Optional[LoyaltyPromotionStatus]
            The status to filter the results by. If a status is provided, only loyalty promotions
            with the specified status are returned. Otherwise, all loyalty promotions associated with
            the loyalty program are returned.

        cursor : typing.Optional[str]
            The cursor returned in the paged response from the previous call to this endpoint.
            Provide this cursor to retrieve the next page of results for your original request.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The maximum number of results to return in a single paged response.
            The minimum value is 1 and the maximum value is 30. The default value is 30.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[LoyaltyPromotion]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.loyalty.programs.promotions.list(
                program_id="program_id",
            )
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            f"v2/loyalty/programs/{jsonable_encoder(program_id)}/promotions",
            method="GET",
            params={
                "status": status,
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListLoyaltyPromotionsResponse,
                    construct_type(
                        type_=ListLoyaltyPromotionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    program_id,
                    status=status,
                    cursor=_parsed_next,
                    limit=limit,
                    request_options=request_options,
                )
                _items = _parsed_response.loyalty_promotions
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        program_id: str,
        *,
        loyalty_promotion: LoyaltyPromotionParams,
        idempotency_key: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateLoyaltyPromotionResponse:
        """
        Creates a loyalty promotion for a [loyalty program](entity:LoyaltyProgram). A loyalty promotion
        enables buyers to earn points in addition to those earned from the base loyalty program.

        This endpoint sets the loyalty promotion to the `ACTIVE` or `SCHEDULED` status, depending on the
        `available_time` setting. A loyalty program can have a maximum of 10 loyalty promotions with an
        `ACTIVE` or `SCHEDULED` status.

        Parameters
        ----------
        program_id : str
            The ID of the [loyalty program](entity:LoyaltyProgram) to associate with the promotion.
            To get the program ID, call [RetrieveLoyaltyProgram](api-endpoint:Loyalty-RetrieveLoyaltyProgram)
            using the `main` keyword.

        loyalty_promotion : LoyaltyPromotionParams
            The loyalty promotion to create.

        idempotency_key : str
            A unique identifier for this request, which is used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateLoyaltyPromotionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.programs.promotions.create(
                program_id="program_id",
                loyalty_promotion={
                    "name": "Tuesday Happy Hour Promo",
                    "incentive": {
                        "type": "POINTS_MULTIPLIER",
                        "points_multiplier_data": {"multiplier": "3.0"},
                    },
                    "available_time": {
                        "time_periods": [
                            "BEGIN:VEVENT\nDTSTART:20220816T160000\nDURATION:PT2H\nRRULE:FREQ=WEEKLY;BYDAY=TU\nEND:VEVENT"
                        ]
                    },
                    "trigger_limit": {"times": 1, "interval": "DAY"},
                    "minimum_spend_amount_money": {"amount": 2000, "currency": "USD"},
                    "qualifying_category_ids": ["XTQPYLR3IIU9C44VRCB3XD12"],
                },
                idempotency_key="ec78c477-b1c3-4899-a209-a4e71337c996",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            program_id,
            loyalty_promotion=loyalty_promotion,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return response.data

    async def get(
        self, promotion_id: str, program_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetLoyaltyPromotionResponse:
        """
        Retrieves a loyalty promotion.

        Parameters
        ----------
        promotion_id : str
            The ID of the [loyalty promotion](entity:LoyaltyPromotion) to retrieve.

        program_id : str
            The ID of the base [loyalty program](entity:LoyaltyProgram). To get the program ID,
            call [RetrieveLoyaltyProgram](api-endpoint:Loyalty-RetrieveLoyaltyProgram) using the `main` keyword.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetLoyaltyPromotionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.programs.promotions.get(
                promotion_id="promotion_id",
                program_id="program_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(promotion_id, program_id, request_options=request_options)
        return response.data

    async def cancel(
        self, promotion_id: str, program_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CancelLoyaltyPromotionResponse:
        """
        Cancels a loyalty promotion. Use this endpoint to cancel an `ACTIVE` promotion earlier than the
        end date, cancel an `ACTIVE` promotion when an end date is not specified, or cancel a `SCHEDULED` promotion.
        Because updating a promotion is not supported, you can also use this endpoint to cancel a promotion before
        you create a new one.

        This endpoint sets the loyalty promotion to the `CANCELED` state

        Parameters
        ----------
        promotion_id : str
            The ID of the [loyalty promotion](entity:LoyaltyPromotion) to cancel. You can cancel a
            promotion that has an `ACTIVE` or `SCHEDULED` status.

        program_id : str
            The ID of the base [loyalty program](entity:LoyaltyProgram).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CancelLoyaltyPromotionResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.programs.promotions.cancel(
                promotion_id="promotion_id",
                program_id="program_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.cancel(promotion_id, program_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .day_of_week import DayOfWeek
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogAvailabilityPeriod(UncheckedBaseModel):
    """
    Represents a time period of availability.
    """

    start_local_time: typing.Optional[str] = pydantic.Field(default=None)
    """
    The start time of an availability period, specified in local time using partial-time
    RFC 3339 format. For example, `8:30:00` for a period starting at 8:30 in the morning.
    Note that the seconds value is always :00, but it is appended for conformance to the RFC.
    """

    end_local_time: typing.Optional[str] = pydantic.Field(default=None)
    """
    The end time of an availability period, specified in local time using partial-time
    RFC 3339 format. For example, `21:00:00` for a period ending at 9:00 in the evening.
    Note that the seconds value is always :00, but it is appended for conformance to the RFC.
    """

    day_of_week: typing.Optional[DayOfWeek] = pydantic.Field(default=None)
    """
    The day of the week for this availability period.
    See [DayOfWeek](#type-dayofweek) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

LoyaltyProgramAccrualRuleType = typing.Union[typing.Literal["VISIT", "SPEND", "ITEM_VARIATION", "CATEGORY"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .subscription import Subscription
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchSubscriptionsResponse(UncheckedBaseModel):
    """
    Defines output parameters in a response from the
    [SearchSubscriptions](api-endpoint:Subscriptions-SearchSubscriptions) endpoint.
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Errors encountered during the request.
    """

    subscriptions: typing.Optional[typing.List[Subscription]] = pydantic.Field(default=None)
    """
    The subscriptions matching the specified query expressions.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    When the total number of resulting subscription exceeds the limit of a paged response, 
    the response includes a cursor for you to use in a subsequent request to fetch the next set of results.
    If the cursor is unset, the response contains the last page of the results.
    
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.quick_pay import QuickPayParams
from ...requests.order import OrderParams
from ...requests.checkout_options import CheckoutOptionsParams
from ...requests.pre_populated_data import PrePopulatedDataParams
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.create_payment_link_response import CreatePaymentLinkResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.get_payment_link_response import GetPaymentLinkResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...requests.payment_link import PaymentLinkParams
from ...types.update_payment_link_response import UpdatePaymentLinkResponse
from ...types.delete_payment_link_response import DeletePaymentLinkResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawPaymentLinksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        idempotency_key: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        quick_pay: typing.Optional[QuickPayParams] = OMIT,
        order: typing.Optional[OrderParams] = OMIT,
        checkout_options: typing.Optional[CheckoutOptionsParams] = OMIT,
        pre_populated_data: typing.Optional[PrePopulatedDataParams] = OMIT,
        payment_note: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreatePaymentLinkResponse]:
        """
        Creates a Square-hosted checkout page. Applications can share the resulting payment link with their buyer to pay for goods and services.

        Parameters
        ----------
        idempotency_key : typing.Optional[str]
            A unique string that identifies this `CreatePaymentLinkRequest` request.
            If you do not provide a unique string (or provide an empty string as the value),
            the endpoint treats each request as independent.

            For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        description : typing.Optional[str]
            A description of the payment link. You provide this optional description that is useful in your
            application context. It is not used anywhere.

        quick_pay : typing.Optional[QuickPayParams]
            Describes an ad hoc item and price for which to generate a quick pay checkout link.
            For more information,
            see [Quick Pay Checkout](https://developer.squareup.com/docs/checkout-api/quick-pay-checkout).

        order : typing.Optional[OrderParams]
            Describes the `Order` for which to create a checkout link.
            For more information,
            see [Square Order Checkout](https://developer.squareup.com/docs/checkout-api/square-order-checkout).

        checkout_options : typing.Optional[CheckoutOptionsParams]
            Describes optional fields to add to the resulting checkout page.
            For more information,
            see [Optional Checkout Configurations](https://developer.squareup.com/docs/checkout-api/optional-checkout-configurations).

        pre_populated_data : typing.Optional[PrePopulatedDataParams]
            Describes fields to prepopulate in the resulting checkout page.
            For more information, see [Prepopulate the shipping address](https://developer.squareup.com/docs/checkout-api/optional-checkout-configurations#prepopulate-the-shipping-address).

        payment_note : typing.Optional[str]
            A note for the payment. After processing the payment, Square adds this note to the resulting `Payment`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreatePaymentLinkResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/online-checkout/payment-links",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "description": description,
                "quick_pay": convert_and_respect_annotation_metadata(
                    object_=quick_pay, annotation=QuickPayParams, direction="write"
                ),
                "order": convert_and_respect_annotation_metadata(
                    object_=order, annotation=OrderParams, direction="write"
                ),
                "checkout_options": convert_and_respect_annotation_metadata(
                    object_=checkout_options, annotation=CheckoutOptionsParams, direction="write"
                ),
                "pre_populated_data": convert_and_respect_annotation_metadata(
                    object_=pre_populated_data, annotation=PrePopulatedDataParams, direction="write"
                ),
                "payment_note": payment_note,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreatePaymentLinkResponse,
                    construct_type(
                        type_=CreatePaymentLinkResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetPaymentLinkResponse]:
        """
        Retrieves a payment link.

        Parameters
        ----------
        id : str
            The ID of link to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetPaymentLinkResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/online-checkout/payment-links/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetPaymentLinkResponse,
                    construct_type(
                        type_=GetPaymentLinkResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self, id: str, *, payment_link: PaymentLinkParams, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[UpdatePaymentLinkResponse]:
        """
        Updates a payment link. You can update the `payment_link` fields such as
        `description`, `checkout_options`, and  `pre_populated_data`.
        You cannot update other fields such as the `order_id`, `version`, `URL`, or `timestamp` field.

        Parameters
        ----------
        id : str
            The ID of the payment link to update.

        payment_link : PaymentLinkParams
            The `payment_link` object describing the updates to apply.
            For more information, see [Update a payment link](https://developer.squareup.com/docs/checkout-api/manage-checkout#update-a-payment-link).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdatePaymentLinkResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/online-checkout/payment-links/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "payment_link": convert_and_respect_annotation_metadata(
                    object_=payment_link, annotation=PaymentLinkParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdatePaymentLinkResponse,
                    construct_type(
                        type_=UpdatePaymentLinkResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeletePaymentLinkResponse]:
        """
        Deletes a payment link.

        Parameters
        ----------
        id : str
            The ID of the payment link to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeletePaymentLinkResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/online-checkout/payment-links/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeletePaymentLinkResponse,
                    construct_type(
                        type_=DeletePaymentLinkResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawPaymentLinksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        idempotency_key: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        quick_pay: typing.Optional[QuickPayParams] = OMIT,
        order: typing.Optional[OrderParams] = OMIT,
        checkout_options: typing.Optional[CheckoutOptionsParams] = OMIT,
        pre_populated_data: typing.Optional[PrePopulatedDataParams] = OMIT,
        payment_note: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreatePaymentLinkResponse]:
        """
        Creates a Square-hosted checkout page. Applications can share the resulting payment link with their buyer to pay for goods and services.

        Parameters
        ----------
        idempotency_key : typing.Optional[str]
            A unique string that identifies this `CreatePaymentLinkRequest` request.
            If you do not provide a unique string (or provide an empty string as the value),
            the endpoint treats each request as independent.

            For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        description : typing.Optional[str]
            A description of the payment link. You provide this optional description that is useful in your
            application context. It is not used anywhere.

        quick_pay : typing.Optional[QuickPayParams]
            Describes an ad hoc item and price for which to generate a quick pay checkout link.
            For more information,
            see [Quick Pay Checkout](https://developer.squareup.com/docs/checkout-api/quick-pay-checkout).

        order : typing.Optional[OrderParams]
            Describes the `Order` for which to create a checkout link.
            For more information,
            see [Square Order Checkout](https://developer.squareup.com/docs/checkout-api/square-order-checkout).

        checkout_options : typing.Optional[CheckoutOptionsParams]
            Describes optional fields to add to the resulting checkout page.
            For more information,
            see [Optional Checkout Configurations](https://developer.squareup.com/docs/checkout-api/optional-checkout-configurations).

        pre_populated_data : typing.Optional[PrePopulatedDataParams]
            Describes fields to prepopulate in the resulting checkout page.
            For more information, see [Prepopulate the shipping address](https://developer.squareup.com/docs/checkout-api/optional-checkout-configurations#prepopulate-the-shipping-address).

        payment_note : typing.Optional[str]
            A note for the payment. After processing the payment, Square adds this note to the resulting `Payment`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreatePaymentLinkResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/online-checkout/payment-links",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "description": description,
                "quick_pay": convert_and_respect_annotation_metadata(
                    object_=quick_pay, annotation=QuickPayParams, direction="write"
                ),
                "order": convert_and_respect_annotation_metadata(
                    object_=order, annotation=OrderParams, direction="write"
                ),
                "checkout_options": convert_and_respect_annotation_metadata(
                    object_=checkout_options, annotation=CheckoutOptionsParams, direction="write"
                ),
                "pre_populated_data": convert_and_respect_annotation_metadata(
                    object_=pre_populated_data, annotation=PrePopulatedDataParams, direction="write"
                ),
                "payment_note": payment_note,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreatePaymentLinkResponse,
                    construct_type(
                        type_=CreatePaymentLinkResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetPaymentLinkResponse]:
        """
        Retrieves a payment link.

        Parameters
        ----------
        id : str
            The ID of link to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetPaymentLinkResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/online-checkout/payment-links/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetPaymentLinkResponse,
                    construct_type(
                        type_=GetPaymentLinkResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self, id: str, *, payment_link: PaymentLinkParams, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[UpdatePaymentLinkResponse]:
        """
        Updates a payment link. You can update the `payment_link` fields such as
        `description`, `checkout_options`, and  `pre_populated_data`.
        You cannot update other fields such as the `order_id`, `version`, `URL`, or `timestamp` field.

        Parameters
        ----------
        id : str
            The ID of the payment link to update.

        payment_link : PaymentLinkParams
            The `payment_link` object describing the updates to apply.
            For more information, see [Update a payment link](https://developer.squareup.com/docs/checkout-api/manage-checkout#update-a-payment-link).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdatePaymentLinkResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/online-checkout/payment-links/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "payment_link": convert_and_respect_annotation_metadata(
                    object_=payment_link, annotation=PaymentLinkParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdatePaymentLinkResponse,
                    construct_type(
                        type_=UpdatePaymentLinkResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeletePaymentLinkResponse]:
        """
        Deletes a payment link.

        Parameters
        ----------
        id : str
            The ID of the payment link to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeletePaymentLinkResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/online-checkout/payment-links/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeletePaymentLinkResponse,
                    construct_type(
                        type_=DeletePaymentLinkResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams


class ModifierLocationOverridesParams(typing_extensions.TypedDict):
    """
    Location-specific overrides for specified properties of a `CatalogModifier` object.
    """

    location_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the `Location` object representing the location. This can include a deactivated location.
    """

    price_money: typing_extensions.NotRequired[MoneyParams]
    """
    The overridden price at the specified location. If this is unspecified, the modifier price is not overridden.
    The modifier becomes free of charge at the specified location, when this `price_money` field is set to 0.
    """

    sold_out: typing_extensions.NotRequired[bool]
    """
    Indicates whether the modifier is sold out at the specified location or not. As an example, for cheese (modifier) burger (item), when the modifier is sold out, it is the cheese, but not the burger, that is sold out.
    The seller can manually set this sold out status. Attempts by an application to set this attribute are ignored.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .team_member_wage import TeamMemberWageParams
from .error import ErrorParams


class ListTeamMemberWagesResponseParams(typing_extensions.TypedDict):
    """
    The response to a request for a set of `TeamMemberWage` objects. The response contains
    a set of `TeamMemberWage` objects.
    """

    team_member_wages: typing_extensions.NotRequired[typing.Sequence[TeamMemberWageParams]]
    """
    A page of `TeamMemberWage` results.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The value supplied in the subsequent request to fetch the next page
    of `TeamMemberWage` results.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .team_member import TeamMember
import pydantic
from .error import Error
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchTeamMembersResponse(UncheckedBaseModel):
    """
    Represents a response from a search request containing a filtered list of `TeamMember` objects.
    """

    team_members: typing.Optional[typing.List[TeamMember]] = pydantic.Field(default=None)
    """
    The filtered list of `TeamMember` objects.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    The opaque cursor for fetching the next page. For more information, see
    [pagination](https://developer.squareup.com/docs/working-with-apis/pagination).
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    The errors that occurred during the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.custom_attribute_definition import CustomAttributeDefinitionParams
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.create_booking_custom_attribute_definition_response import CreateBookingCustomAttributeDefinitionResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.retrieve_booking_custom_attribute_definition_response import (
    RetrieveBookingCustomAttributeDefinitionResponse,
)
from ...core.jsonable_encoder import jsonable_encoder
from ...types.update_booking_custom_attribute_definition_response import UpdateBookingCustomAttributeDefinitionResponse
from ...types.delete_booking_custom_attribute_definition_response import DeleteBookingCustomAttributeDefinitionResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCustomAttributeDefinitionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateBookingCustomAttributeDefinitionResponse]:
        """
        Creates a bookings custom attribute definition.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition to create, with the following fields:

            - `key`

            - `name`. If provided, `name` must be unique (case-sensitive) across all visible booking-related custom attribute
            definitions for the seller.

            - `description`

            - `visibility`. Note that all custom attributes are visible in exported booking data, including those set to
            `VISIBILITY_HIDDEN`.

            - `schema`. With the exception of the `Selection` data type, the `schema` is specified as a
            simple URL to the JSON schema definition hosted on the Square CDN. For more information, see
            [Specifying the schema](https://developer.squareup.com/docs/booking-custom-attributes-api/custom-attribute-definitions#specify-schema).

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateBookingCustomAttributeDefinitionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/bookings/custom-attribute-definitions",
            method="POST",
            json={
                "custom_attribute_definition": convert_and_respect_annotation_metadata(
                    object_=custom_attribute_definition, annotation=CustomAttributeDefinitionParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateBookingCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=CreateBookingCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, key: str, *, version: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RetrieveBookingCustomAttributeDefinitionResponse]:
        """
        Retrieves a bookings custom attribute definition.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to retrieve. If the requesting application
            is not the definition owner, you must use the qualified key.

        version : typing.Optional[int]
            The current version of the custom attribute definition, which is used for strongly consistent
            reads to guarantee that you receive the most up-to-date data. When included in the request,
            Square returns the specified version or a higher version if one exists. If the specified version
            is higher than the current version, Square returns a `BAD_REQUEST` error.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RetrieveBookingCustomAttributeDefinitionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/bookings/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="GET",
            params={
                "version": version,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveBookingCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=RetrieveBookingCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        key: str,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateBookingCustomAttributeDefinitionResponse]:
        """
        Updates a bookings custom attribute definition.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to update.

        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition that contains the fields to update. Only the following fields can be updated:
            - `name`
            - `description`
            - `visibility`
            - `schema` for a `Selection` data type. Only changes to the named options or the maximum number of allowed
            selections are supported.

            For more information, see
            [Updatable definition fields](https://developer.squareup.com/docs/booking-custom-attributes-api/custom-attribute-definitions#updatable-definition-fields).

            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include the optional `version` field and specify the current version of the custom attribute definition.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateBookingCustomAttributeDefinitionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/bookings/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="PUT",
            json={
                "custom_attribute_definition": convert_and_respect_annotation_metadata(
                    object_=custom_attribute_definition, annotation=CustomAttributeDefinitionParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateBookingCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=UpdateBookingCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteBookingCustomAttributeDefinitionResponse]:
        """
        Deletes a bookings custom attribute definition.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteBookingCustomAttributeDefinitionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/bookings/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteBookingCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=DeleteBookingCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawCustomAttributeDefinitionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateBookingCustomAttributeDefinitionResponse]:
        """
        Creates a bookings custom attribute definition.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition to create, with the following fields:

            - `key`

            - `name`. If provided, `name` must be unique (case-sensitive) across all visible booking-related custom attribute
            definitions for the seller.

            - `description`

            - `visibility`. Note that all custom attributes are visible in exported booking data, including those set to
            `VISIBILITY_HIDDEN`.

            - `schema`. With the exception of the `Selection` data type, the `schema` is specified as a
            simple URL to the JSON schema definition hosted on the Square CDN. For more information, see
            [Specifying the schema](https://developer.squareup.com/docs/booking-custom-attributes-api/custom-attribute-definitions#specify-schema).

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateBookingCustomAttributeDefinitionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/bookings/custom-attribute-definitions",
            method="POST",
            json={
                "custom_attribute_definition": convert_and_respect_annotation_metadata(
                    object_=custom_attribute_definition, annotation=CustomAttributeDefinitionParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateBookingCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=CreateBookingCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, key: str, *, version: typing.Optional[int] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RetrieveBookingCustomAttributeDefinitionResponse]:
        """
        Retrieves a bookings custom attribute definition.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to retrieve. If the requesting application
            is not the definition owner, you must use the qualified key.

        version : typing.Optional[int]
            The current version of the custom attribute definition, which is used for strongly consistent
            reads to guarantee that you receive the most up-to-date data. When included in the request,
            Square returns the specified version or a higher version if one exists. If the specified version
            is higher than the current version, Square returns a `BAD_REQUEST` error.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RetrieveBookingCustomAttributeDefinitionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/bookings/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="GET",
            params={
                "version": version,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveBookingCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=RetrieveBookingCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        key: str,
        *,
        custom_attribute_definition: CustomAttributeDefinitionParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateBookingCustomAttributeDefinitionResponse]:
        """
        Updates a bookings custom attribute definition.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to update.

        custom_attribute_definition : CustomAttributeDefinitionParams
            The custom attribute definition that contains the fields to update. Only the following fields can be updated:
            - `name`
            - `description`
            - `visibility`
            - `schema` for a `Selection` data type. Only changes to the named options or the maximum number of allowed
            selections are supported.

            For more information, see
            [Updatable definition fields](https://developer.squareup.com/docs/booking-custom-attributes-api/custom-attribute-definitions#updatable-definition-fields).

            To enable [optimistic concurrency](https://developer.squareup.com/docs/build-basics/common-api-patterns/optimistic-concurrency)
            control, include the optional `version` field and specify the current version of the custom attribute definition.

        idempotency_key : typing.Optional[str]
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateBookingCustomAttributeDefinitionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/bookings/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="PUT",
            json={
                "custom_attribute_definition": convert_and_respect_annotation_metadata(
                    object_=custom_attribute_definition, annotation=CustomAttributeDefinitionParams, direction="write"
                ),
                "idempotency_key": idempotency_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateBookingCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=UpdateBookingCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, key: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteBookingCustomAttributeDefinitionResponse]:
        """
        Deletes a bookings custom attribute definition.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        key : str
            The key of the custom attribute definition to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteBookingCustomAttributeDefinitionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/bookings/custom-attribute-definitions/{jsonable_encoder(key)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteBookingCustomAttributeDefinitionResponse,
                    construct_type(
                        type_=DeleteBookingCustomAttributeDefinitionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .checkout_location_settings_branding_header_type import CheckoutLocationSettingsBrandingHeaderType
import pydantic
from .checkout_location_settings_branding_button_shape import CheckoutLocationSettingsBrandingButtonShape
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CheckoutLocationSettingsBranding(UncheckedBaseModel):
    header_type: typing.Optional[CheckoutLocationSettingsBrandingHeaderType] = pydantic.Field(default=None)
    """
    Show the location logo on the checkout page.
    See [HeaderType](#type-headertype) for possible values
    """

    button_color: typing.Optional[str] = pydantic.Field(default=None)
    """
    The HTML-supported hex color for the button on the checkout page (for example, "#FFFFFF").
    """

    button_shape: typing.Optional[CheckoutLocationSettingsBrandingButtonShape] = pydantic.Field(default=None)
    """
    The shape of the button on the checkout page.
    See [ButtonShape](#type-buttonshape) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .booking import BookingParams
from .error import ErrorParams


class ListBookingsResponseParams(typing_extensions.TypedDict):
    bookings: typing_extensions.NotRequired[typing.Sequence[BookingParams]]
    """
    The list of targeted bookings.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor to be used in the subsequent request to get the next page of the results. Stop retrieving the next page of the results when the cursor is not set.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class OrderLineItemPricingBlocklistsBlockedTaxParams(typing_extensions.TypedDict):
    """
    A tax to block from applying to a line item. The tax must be
    identified by either `tax_uid` or `tax_catalog_object_id`, but not both.
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID of the `BlockedTax` within the order.
    """

    tax_uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The `uid` of the tax that should be blocked. Use this field to block
    ad hoc taxes. For catalog, taxes use the `tax_catalog_object_id` field.
    """

    tax_catalog_object_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The `catalog_object_id` of the tax that should be blocked.
    Use this field to block catalog taxes. For ad hoc taxes, use the
    `tax_uid` field.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CardPaymentTimelineParams(typing_extensions.TypedDict):
    """
    The timeline for card payments.
    """

    authorized_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The timestamp when the payment was authorized, in RFC 3339 format.
    """

    captured_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The timestamp when the payment was captured, in RFC 3339 format.
    """

    voided_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The timestamp when the payment was voided, in RFC 3339 format.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .product_type import ProductType
from .device_code_status import DeviceCodeStatus
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class DeviceCode(UncheckedBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique id for this device code.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional user-defined name for the device code.
    """

    code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique code that can be used to login.
    """

    device_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique id of the device that used this code. Populated when the device is paired up.
    """

    product_type: ProductType = pydantic.Field(default="TERMINAL_API")
    """
    The targeting product type of the device code.
    """

    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The location assigned to this code.
    """

    status: typing.Optional[DeviceCodeStatus] = pydantic.Field(default=None)
    """
    The pairing status of the device code.
    See [DeviceCodeStatus](#type-devicecodestatus) for possible values
    """

    pair_by: typing.Optional[str] = pydantic.Field(default=None)
    """
    When this DeviceCode will expire and no longer login. Timestamp in RFC 3339 format.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    When this DeviceCode was created. Timestamp in RFC 3339 format.
    """

    status_changed_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    When this DeviceCode's status was last changed. Timestamp in RFC 3339 format.
    """

    paired_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    When this DeviceCode was paired. Timestamp in RFC 3339 format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .loyalty_account import LoyaltyAccountParams


class SearchLoyaltyAccountsResponseParams(typing_extensions.TypedDict):
    """
    A response that includes loyalty accounts that satisfy the search criteria.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    loyalty_accounts: typing_extensions.NotRequired[typing.Sequence[LoyaltyAccountParams]]
    """
    The loyalty accounts that met the search criteria,  
    in order of creation date.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor to use in a subsequent 
    request. If empty, this is the final response.
    For more information, 
    see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .invoice_payment_reminder_status import InvoicePaymentReminderStatus
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class InvoicePaymentReminder(UncheckedBaseModel):
    """
    Describes a payment request reminder (automatic notification) that Square sends
    to the customer. You configure a reminder relative to the payment request
    `due_date`.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A Square-assigned ID that uniquely identifies the reminder within the
    `InvoicePaymentRequest`.
    """

    relative_scheduled_days: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of days before (a negative number) or after (a positive number)
    the payment request `due_date` when the reminder is sent. For example, -3 indicates that
    the reminder should be sent 3 days before the payment request `due_date`.
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    The reminder message.
    """

    status: typing.Optional[InvoicePaymentReminderStatus] = pydantic.Field(default=None)
    """
    The status of the reminder.
    See [InvoicePaymentReminderStatus](#type-invoicepaymentreminderstatus) for possible values
    """

    sent_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    If sent, the timestamp when the reminder was sent, in RFC 3339 format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.subscription_action_type import SubscriptionActionType
import typing
from .phase import PhaseParams


class SubscriptionActionParams(typing_extensions.TypedDict):
    """
    Represents an action as a pending change to a subscription.
    """

    id: typing_extensions.NotRequired[str]
    """
    The ID of an action scoped to a subscription.
    """

    type: typing_extensions.NotRequired[SubscriptionActionType]
    """
    The type of the action.
    See [SubscriptionActionType](#type-subscriptionactiontype) for possible values
    """

    effective_date: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The `YYYY-MM-DD`-formatted date when the action occurs on the subscription.
    """

    monthly_billing_anchor_date: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The new billing anchor day value, for a `CHANGE_BILLING_ANCHOR_DATE` action.
    """

    phases: typing_extensions.NotRequired[typing.Optional[typing.Sequence[PhaseParams]]]
    """
    A list of Phases, to pass phase-specific information used in the swap.
    """

    new_plan_variation_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The target subscription plan variation that a subscription switches to, for a `SWAP_PLAN` action.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawWageSettingClient
from ...core.request_options import RequestOptions
from ...types.get_wage_setting_response import GetWageSettingResponse
from ...requests.wage_setting import WageSettingParams
from ...types.update_wage_setting_response import UpdateWageSettingResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawWageSettingClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class WageSettingClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawWageSettingClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawWageSettingClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawWageSettingClient
        """
        return self._raw_client

    def get(
        self, team_member_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetWageSettingResponse:
        """
        Retrieves a `WageSetting` object for a team member specified
        by `TeamMember.id`. For more information, see
        [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#retrievewagesetting).

        Square recommends using [RetrieveTeamMember](api-endpoint:Team-RetrieveTeamMember) or [SearchTeamMembers](api-endpoint:Team-SearchTeamMembers)
        to get this information directly from the `TeamMember.wage_setting` field.

        Parameters
        ----------
        team_member_id : str
            The ID of the team member for which to retrieve the wage setting.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetWageSettingResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.team_members.wage_setting.get(
            team_member_id="team_member_id",
        )
        """
        response = self._raw_client.get(team_member_id, request_options=request_options)
        return response.data

    def update(
        self,
        team_member_id: str,
        *,
        wage_setting: WageSettingParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateWageSettingResponse:
        """
        Creates or updates a `WageSetting` object. The object is created if a
        `WageSetting` with the specified `team_member_id` doesn't exist. Otherwise,
        it fully replaces the `WageSetting` object for the team member.
        The `WageSetting` is returned on a successful update. For more information, see
        [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#create-or-update-a-wage-setting).

        Square recommends using [CreateTeamMember](api-endpoint:Team-CreateTeamMember) or [UpdateTeamMember](api-endpoint:Team-UpdateTeamMember)
        to manage the `TeamMember.wage_setting` field directly.

        Parameters
        ----------
        team_member_id : str
            The ID of the team member for which to update the `WageSetting` object.

        wage_setting : WageSettingParams
            The complete `WageSetting` object. For all job assignments, specify one of the following:
            - `job_id` (recommended) - If needed, call [ListJobs](api-endpoint:Team-ListJobs) to get a list of all jobs.
            Requires Square API version 2024-12-18 or later.
            - `job_title` - Use the exact, case-sensitive spelling of an existing title unless you want to create a new job.
            This value is ignored if `job_id` is also provided.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateWageSettingResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.team_members.wage_setting.update(
            team_member_id="team_member_id",
            wage_setting={
                "job_assignments": [
                    {
                        "job_title": "Manager",
                        "pay_type": "SALARY",
                        "annual_rate": {"amount": 3000000, "currency": "USD"},
                        "weekly_hours": 40,
                    },
                    {
                        "job_title": "Cashier",
                        "pay_type": "HOURLY",
                        "hourly_rate": {"amount": 2000, "currency": "USD"},
                    },
                ],
                "is_overtime_exempt": True,
            },
        )
        """
        response = self._raw_client.update(team_member_id, wage_setting=wage_setting, request_options=request_options)
        return response.data


class AsyncWageSettingClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawWageSettingClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawWageSettingClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawWageSettingClient
        """
        return self._raw_client

    async def get(
        self, team_member_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetWageSettingResponse:
        """
        Retrieves a `WageSetting` object for a team member specified
        by `TeamMember.id`. For more information, see
        [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#retrievewagesetting).

        Square recommends using [RetrieveTeamMember](api-endpoint:Team-RetrieveTeamMember) or [SearchTeamMembers](api-endpoint:Team-SearchTeamMembers)
        to get this information directly from the `TeamMember.wage_setting` field.

        Parameters
        ----------
        team_member_id : str
            The ID of the team member for which to retrieve the wage setting.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetWageSettingResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.team_members.wage_setting.get(
                team_member_id="team_member_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(team_member_id, request_options=request_options)
        return response.data

    async def update(
        self,
        team_member_id: str,
        *,
        wage_setting: WageSettingParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateWageSettingResponse:
        """
        Creates or updates a `WageSetting` object. The object is created if a
        `WageSetting` with the specified `team_member_id` doesn't exist. Otherwise,
        it fully replaces the `WageSetting` object for the team member.
        The `WageSetting` is returned on a successful update. For more information, see
        [Troubleshooting the Team API](https://developer.squareup.com/docs/team/troubleshooting#create-or-update-a-wage-setting).

        Square recommends using [CreateTeamMember](api-endpoint:Team-CreateTeamMember) or [UpdateTeamMember](api-endpoint:Team-UpdateTeamMember)
        to manage the `TeamMember.wage_setting` field directly.

        Parameters
        ----------
        team_member_id : str
            The ID of the team member for which to update the `WageSetting` object.

        wage_setting : WageSettingParams
            The complete `WageSetting` object. For all job assignments, specify one of the following:
            - `job_id` (recommended) - If needed, call [ListJobs](api-endpoint:Team-ListJobs) to get a list of all jobs.
            Requires Square API version 2024-12-18 or later.
            - `job_title` - Use the exact, case-sensitive spelling of an existing title unless you want to create a new job.
            This value is ignored if `job_id` is also provided.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateWageSettingResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.team_members.wage_setting.update(
                team_member_id="team_member_id",
                wage_setting={
                    "job_assignments": [
                        {
                            "job_title": "Manager",
                            "pay_type": "SALARY",
                            "annual_rate": {"amount": 3000000, "currency": "USD"},
                            "weekly_hours": 40,
                        },
                        {
                            "job_title": "Cashier",
                            "pay_type": "HOURLY",
                            "hourly_rate": {"amount": 2000, "currency": "USD"},
                        },
                    ],
                    "is_overtime_exempt": True,
                },
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(
            team_member_id, wage_setting=wage_setting, request_options=request_options
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.inventory_change_type import InventoryChangeType
from .inventory_physical_count import InventoryPhysicalCountParams
from .inventory_adjustment import InventoryAdjustmentParams
from .inventory_transfer import InventoryTransferParams
from .catalog_measurement_unit import CatalogMeasurementUnitParams


class InventoryChangeParams(typing_extensions.TypedDict):
    """
    Represents a single physical count, inventory, adjustment, or transfer
    that is part of the history of inventory changes for a particular
    [CatalogObject](entity:CatalogObject) instance.
    """

    type: typing_extensions.NotRequired[InventoryChangeType]
    """
    Indicates how the inventory change is applied. See
    [InventoryChangeType](entity:InventoryChangeType) for all possible values.
    See [InventoryChangeType](#type-inventorychangetype) for possible values
    """

    physical_count: typing_extensions.NotRequired[InventoryPhysicalCountParams]
    """
    Contains details about the physical count when `type` is
    `PHYSICAL_COUNT`, and is unset for all other change types.
    """

    adjustment: typing_extensions.NotRequired[InventoryAdjustmentParams]
    """
    Contains details about the inventory adjustment when `type` is
    `ADJUSTMENT`, and is unset for all other change types.
    """

    transfer: typing_extensions.NotRequired[InventoryTransferParams]
    """
    Contains details about the inventory transfer when `type` is
    `TRANSFER`, and is unset for all other change types.
    
    _Note:_ An [InventoryTransfer](entity:InventoryTransfer) object can only be set in the input to the
    [BatchChangeInventory](api-endpoint:Inventory-BatchChangeInventory) endpoint when the seller has an active Retail Plus subscription.
    """

    measurement_unit: typing_extensions.NotRequired[CatalogMeasurementUnitParams]
    """
    The [CatalogMeasurementUnit](entity:CatalogMeasurementUnit) object representing the catalog measurement unit associated with the inventory change.
    """

    measurement_unit_id: typing_extensions.NotRequired[str]
    """
    The ID of the [CatalogMeasurementUnit](entity:CatalogMeasurementUnit) object representing the catalog measurement unit associated with the inventory change.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .inventory_change_type import InventoryChangeType
import pydantic
from .inventory_physical_count import InventoryPhysicalCount
from .inventory_adjustment import InventoryAdjustment
from .inventory_transfer import InventoryTransfer
from .catalog_measurement_unit import CatalogMeasurementUnit
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class InventoryChange(UncheckedBaseModel):
    """
    Represents a single physical count, inventory, adjustment, or transfer
    that is part of the history of inventory changes for a particular
    [CatalogObject](entity:CatalogObject) instance.
    """

    type: typing.Optional[InventoryChangeType] = pydantic.Field(default=None)
    """
    Indicates how the inventory change is applied. See
    [InventoryChangeType](entity:InventoryChangeType) for all possible values.
    See [InventoryChangeType](#type-inventorychangetype) for possible values
    """

    physical_count: typing.Optional[InventoryPhysicalCount] = pydantic.Field(default=None)
    """
    Contains details about the physical count when `type` is
    `PHYSICAL_COUNT`, and is unset for all other change types.
    """

    adjustment: typing.Optional[InventoryAdjustment] = pydantic.Field(default=None)
    """
    Contains details about the inventory adjustment when `type` is
    `ADJUSTMENT`, and is unset for all other change types.
    """

    transfer: typing.Optional[InventoryTransfer] = pydantic.Field(default=None)
    """
    Contains details about the inventory transfer when `type` is
    `TRANSFER`, and is unset for all other change types.
    
    _Note:_ An [InventoryTransfer](entity:InventoryTransfer) object can only be set in the input to the
    [BatchChangeInventory](api-endpoint:Inventory-BatchChangeInventory) endpoint when the seller has an active Retail Plus subscription.
    """

    measurement_unit: typing.Optional[CatalogMeasurementUnit] = pydantic.Field(default=None)
    """
    The [CatalogMeasurementUnit](entity:CatalogMeasurementUnit) object representing the catalog measurement unit associated with the inventory change.
    """

    measurement_unit_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the [CatalogMeasurementUnit](entity:CatalogMeasurementUnit) object representing the catalog measurement unit associated with the inventory change.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class DeviceMetadata(UncheckedBaseModel):
    battery_percentage: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Terminal’s remaining battery percentage, between 1-100.
    """

    charging_state: typing.Optional[str] = pydantic.Field(default=None)
    """
    The current charging state of the Terminal.
    Options: `CHARGING`, `NOT_CHARGING`
    """

    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the Square seller business location associated with the Terminal.
    """

    merchant_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the Square merchant account that is currently signed-in to the Terminal.
    """

    network_connection_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Terminal’s current network connection type.
    Options: `WIFI`, `ETHERNET`
    """

    payment_region: typing.Optional[str] = pydantic.Field(default=None)
    """
    The country in which the Terminal is authorized to take payments.
    """

    serial_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier assigned to the Terminal, which can be found on the lower back
    of the device.
    """

    os_version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The current version of the Terminal’s operating system.
    """

    app_version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The current version of the application running on the Terminal.
    """

    wifi_network_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the Wi-Fi network to which the Terminal is connected.
    """

    wifi_network_strength: typing.Optional[str] = pydantic.Field(default=None)
    """
    The signal strength of the Wi-FI network connection.
    Options: `POOR`, `FAIR`, `GOOD`, `EXCELLENT`
    """

    ip_address: typing.Optional[str] = pydantic.Field(default=None)
    """
    The IP address of the Terminal.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from .environment import SquareEnvironment
import os
import httpx
from .core.client_wrapper import SyncClientWrapper
from .mobile.client import MobileClient
from .o_auth.client import OAuthClient
from .v1transactions.client import V1TransactionsClient
from .apple_pay.client import ApplePayClient
from .bank_accounts.client import BankAccountsClient
from .bookings.client import BookingsClient
from .cards.client import CardsClient
from .catalog.client import CatalogClient
from .customers.client import CustomersClient
from .devices.client import DevicesClient
from .disputes.client import DisputesClient
from .employees.client import EmployeesClient
from .events.client import EventsClient
from .gift_cards.client import GiftCardsClient
from .inventory.client import InventoryClient
from .invoices.client import InvoicesClient
from .labor.client import LaborClient
from .locations.client import LocationsClient
from .loyalty.client import LoyaltyClient
from .merchants.client import MerchantsClient
from .checkout.client import CheckoutClient
from .orders.client import OrdersClient
from .payments.client import PaymentsClient
from .payouts.client import PayoutsClient
from .refunds.client import RefundsClient
from .sites.client import SitesClient
from .snippets.client import SnippetsClient
from .subscriptions.client import SubscriptionsClient
from .team_members.client import TeamMembersClient
from .team.client import TeamClient
from .terminal.client import TerminalClient
from .vendors.client import VendorsClient
from .cash_drawers.client import CashDrawersClient
from .webhooks.client import WebhooksClient
from .core.client_wrapper import AsyncClientWrapper
from .mobile.client import AsyncMobileClient
from .o_auth.client import AsyncOAuthClient
from .v1transactions.client import AsyncV1TransactionsClient
from .apple_pay.client import AsyncApplePayClient
from .bank_accounts.client import AsyncBankAccountsClient
from .bookings.client import AsyncBookingsClient
from .cards.client import AsyncCardsClient
from .catalog.client import AsyncCatalogClient
from .customers.client import AsyncCustomersClient
from .devices.client import AsyncDevicesClient
from .disputes.client import AsyncDisputesClient
from .employees.client import AsyncEmployeesClient
from .events.client import AsyncEventsClient
from .gift_cards.client import AsyncGiftCardsClient
from .inventory.client import AsyncInventoryClient
from .invoices.client import AsyncInvoicesClient
from .labor.client import AsyncLaborClient
from .locations.client import AsyncLocationsClient
from .loyalty.client import AsyncLoyaltyClient
from .merchants.client import AsyncMerchantsClient
from .checkout.client import AsyncCheckoutClient
from .orders.client import AsyncOrdersClient
from .payments.client import AsyncPaymentsClient
from .payouts.client import AsyncPayoutsClient
from .refunds.client import AsyncRefundsClient
from .sites.client import AsyncSitesClient
from .snippets.client import AsyncSnippetsClient
from .subscriptions.client import AsyncSubscriptionsClient
from .team_members.client import AsyncTeamMembersClient
from .team.client import AsyncTeamClient
from .terminal.client import AsyncTerminalClient
from .vendors.client import AsyncVendorsClient
from .cash_drawers.client import AsyncCashDrawersClient
from .webhooks.client import AsyncWebhooksClient


class Square:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : SquareEnvironment
        The environment to use for requests from the client. from .environment import SquareEnvironment



        Defaults to SquareEnvironment.PRODUCTION



    token : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    version : typing.Optional[str]
    Examples
    --------
    from square import Square

    client = Square(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: SquareEnvironment = SquareEnvironment.PRODUCTION,
        token: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("SQUARE_TOKEN"),
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
        version: typing.Optional[str] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            token=token,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
            version=version,
        )
        self.mobile = MobileClient(client_wrapper=self._client_wrapper)
        self.o_auth = OAuthClient(client_wrapper=self._client_wrapper)
        self.v1transactions = V1TransactionsClient(client_wrapper=self._client_wrapper)
        self.apple_pay = ApplePayClient(client_wrapper=self._client_wrapper)
        self.bank_accounts = BankAccountsClient(client_wrapper=self._client_wrapper)
        self.bookings = BookingsClient(client_wrapper=self._client_wrapper)
        self.cards = CardsClient(client_wrapper=self._client_wrapper)
        self.catalog = CatalogClient(client_wrapper=self._client_wrapper)
        self.customers = CustomersClient(client_wrapper=self._client_wrapper)
        self.devices = DevicesClient(client_wrapper=self._client_wrapper)
        self.disputes = DisputesClient(client_wrapper=self._client_wrapper)
        self.employees = EmployeesClient(client_wrapper=self._client_wrapper)
        self.events = EventsClient(client_wrapper=self._client_wrapper)
        self.gift_cards = GiftCardsClient(client_wrapper=self._client_wrapper)
        self.inventory = InventoryClient(client_wrapper=self._client_wrapper)
        self.invoices = InvoicesClient(client_wrapper=self._client_wrapper)
        self.labor = LaborClient(client_wrapper=self._client_wrapper)
        self.locations = LocationsClient(client_wrapper=self._client_wrapper)
        self.loyalty = LoyaltyClient(client_wrapper=self._client_wrapper)
        self.merchants = MerchantsClient(client_wrapper=self._client_wrapper)
        self.checkout = CheckoutClient(client_wrapper=self._client_wrapper)
        self.orders = OrdersClient(client_wrapper=self._client_wrapper)
        self.payments = PaymentsClient(client_wrapper=self._client_wrapper)
        self.payouts = PayoutsClient(client_wrapper=self._client_wrapper)
        self.refunds = RefundsClient(client_wrapper=self._client_wrapper)
        self.sites = SitesClient(client_wrapper=self._client_wrapper)
        self.snippets = SnippetsClient(client_wrapper=self._client_wrapper)
        self.subscriptions = SubscriptionsClient(client_wrapper=self._client_wrapper)
        self.team_members = TeamMembersClient(client_wrapper=self._client_wrapper)
        self.team = TeamClient(client_wrapper=self._client_wrapper)
        self.terminal = TerminalClient(client_wrapper=self._client_wrapper)
        self.vendors = VendorsClient(client_wrapper=self._client_wrapper)
        self.cash_drawers = CashDrawersClient(client_wrapper=self._client_wrapper)
        self.webhooks = WebhooksClient(client_wrapper=self._client_wrapper)


class AsyncSquare:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : SquareEnvironment
        The environment to use for requests from the client. from .environment import SquareEnvironment



        Defaults to SquareEnvironment.PRODUCTION



    token : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    version : typing.Optional[str]
    Examples
    --------
    from square import AsyncSquare

    client = AsyncSquare(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: SquareEnvironment = SquareEnvironment.PRODUCTION,
        token: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = os.getenv("SQUARE_TOKEN"),
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
        version: typing.Optional[str] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            token=token,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
            version=version,
        )
        self.mobile = AsyncMobileClient(client_wrapper=self._client_wrapper)
        self.o_auth = AsyncOAuthClient(client_wrapper=self._client_wrapper)
        self.v1transactions = AsyncV1TransactionsClient(client_wrapper=self._client_wrapper)
        self.apple_pay = AsyncApplePayClient(client_wrapper=self._client_wrapper)
        self.bank_accounts = AsyncBankAccountsClient(client_wrapper=self._client_wrapper)
        self.bookings = AsyncBookingsClient(client_wrapper=self._client_wrapper)
        self.cards = AsyncCardsClient(client_wrapper=self._client_wrapper)
        self.catalog = AsyncCatalogClient(client_wrapper=self._client_wrapper)
        self.customers = AsyncCustomersClient(client_wrapper=self._client_wrapper)
        self.devices = AsyncDevicesClient(client_wrapper=self._client_wrapper)
        self.disputes = AsyncDisputesClient(client_wrapper=self._client_wrapper)
        self.employees = AsyncEmployeesClient(client_wrapper=self._client_wrapper)
        self.events = AsyncEventsClient(client_wrapper=self._client_wrapper)
        self.gift_cards = AsyncGiftCardsClient(client_wrapper=self._client_wrapper)
        self.inventory = AsyncInventoryClient(client_wrapper=self._client_wrapper)
        self.invoices = AsyncInvoicesClient(client_wrapper=self._client_wrapper)
        self.labor = AsyncLaborClient(client_wrapper=self._client_wrapper)
        self.locations = AsyncLocationsClient(client_wrapper=self._client_wrapper)
        self.loyalty = AsyncLoyaltyClient(client_wrapper=self._client_wrapper)
        self.merchants = AsyncMerchantsClient(client_wrapper=self._client_wrapper)
        self.checkout = AsyncCheckoutClient(client_wrapper=self._client_wrapper)
        self.orders = AsyncOrdersClient(client_wrapper=self._client_wrapper)
        self.payments = AsyncPaymentsClient(client_wrapper=self._client_wrapper)
        self.payouts = AsyncPayoutsClient(client_wrapper=self._client_wrapper)
        self.refunds = AsyncRefundsClient(client_wrapper=self._client_wrapper)
        self.sites = AsyncSitesClient(client_wrapper=self._client_wrapper)
        self.snippets = AsyncSnippetsClient(client_wrapper=self._client_wrapper)
        self.subscriptions = AsyncSubscriptionsClient(client_wrapper=self._client_wrapper)
        self.team_members = AsyncTeamMembersClient(client_wrapper=self._client_wrapper)
        self.team = AsyncTeamClient(client_wrapper=self._client_wrapper)
        self.terminal = AsyncTerminalClient(client_wrapper=self._client_wrapper)
        self.vendors = AsyncVendorsClient(client_wrapper=self._client_wrapper)
        self.cash_drawers = AsyncCashDrawersClient(client_wrapper=self._client_wrapper)
        self.webhooks = AsyncWebhooksClient(client_wrapper=self._client_wrapper)


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: SquareEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")



# This file was auto-generated by Fern from our API Definition.

import typing

TerminalActionActionType = typing.Union[
    typing.Literal["QR_CODE", "PING", "SAVE_CARD", "SIGNATURE", "CONFIRMATION", "RECEIPT", "DATA_COLLECTION", "SELECT"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class UpdateCatalogImageRequestParams(typing_extensions.TypedDict):
    idempotency_key: str
    """
    A unique string that identifies this UpdateCatalogImage request.
    Keys can be any valid string but must be unique for every UpdateCatalogImage request.
    
    See [Idempotency keys](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
from .loyalty_event_type import LoyaltyEventType
import typing
from .loyalty_event_accumulate_points import LoyaltyEventAccumulatePoints
from .loyalty_event_create_reward import LoyaltyEventCreateReward
from .loyalty_event_redeem_reward import LoyaltyEventRedeemReward
from .loyalty_event_delete_reward import LoyaltyEventDeleteReward
from .loyalty_event_adjust_points import LoyaltyEventAdjustPoints
from .loyalty_event_source import LoyaltyEventSource
from .loyalty_event_expire_points import LoyaltyEventExpirePoints
from .loyalty_event_other import LoyaltyEventOther
from .loyalty_event_accumulate_promotion_points import LoyaltyEventAccumulatePromotionPoints
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LoyaltyEvent(UncheckedBaseModel):
    """
    Provides information about a loyalty event.
    For more information, see [Search for Balance-Changing Loyalty Events](https://developer.squareup.com/docs/loyalty-api/loyalty-events).
    """

    id: str = pydantic.Field()
    """
    The Square-assigned ID of the loyalty event.
    """

    type: LoyaltyEventType = pydantic.Field()
    """
    The type of the loyalty event.
    See [LoyaltyEventType](#type-loyaltyeventtype) for possible values
    """

    created_at: str = pydantic.Field()
    """
    The timestamp when the event was created, in RFC 3339 format.
    """

    accumulate_points: typing.Optional[LoyaltyEventAccumulatePoints] = pydantic.Field(default=None)
    """
    Provides metadata when the event `type` is `ACCUMULATE_POINTS`.
    """

    create_reward: typing.Optional[LoyaltyEventCreateReward] = pydantic.Field(default=None)
    """
    Provides metadata when the event `type` is `CREATE_REWARD`.
    """

    redeem_reward: typing.Optional[LoyaltyEventRedeemReward] = pydantic.Field(default=None)
    """
    Provides metadata when the event `type` is `REDEEM_REWARD`.
    """

    delete_reward: typing.Optional[LoyaltyEventDeleteReward] = pydantic.Field(default=None)
    """
    Provides metadata when the event `type` is `DELETE_REWARD`.
    """

    adjust_points: typing.Optional[LoyaltyEventAdjustPoints] = pydantic.Field(default=None)
    """
    Provides metadata when the event `type` is `ADJUST_POINTS`.
    """

    loyalty_account_id: str = pydantic.Field()
    """
    The ID of the [loyalty account](entity:LoyaltyAccount) associated with the event.
    """

    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the [location](entity:Location) where the event occurred.
    """

    source: LoyaltyEventSource = pydantic.Field()
    """
    Defines whether the event was generated by the Square Point of Sale.
    See [LoyaltyEventSource](#type-loyaltyeventsource) for possible values
    """

    expire_points: typing.Optional[LoyaltyEventExpirePoints] = pydantic.Field(default=None)
    """
    Provides metadata when the event `type` is `EXPIRE_POINTS`.
    """

    other_event: typing.Optional[LoyaltyEventOther] = pydantic.Field(default=None)
    """
    Provides metadata when the event `type` is `OTHER`.
    """

    accumulate_promotion_points: typing.Optional[LoyaltyEventAccumulatePromotionPoints] = pydantic.Field(default=None)
    """
    Provides metadata when the event `type` is `ACCUMULATE_PROMOTION_POINTS`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .employee_status import EmployeeStatus
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Employee(UncheckedBaseModel):
    """
    An employee object that is used by the external API.

    DEPRECATED at version 2020-08-26. Replaced by [TeamMember](entity:TeamMember).
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    UUID for this object.
    """

    first_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The employee's first name.
    """

    last_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The employee's last name.
    """

    email: typing.Optional[str] = pydantic.Field(default=None)
    """
    The employee's email address
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    The employee's phone number in E.164 format, i.e. "+12125554250"
    """

    location_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of location IDs where this employee has access to.
    """

    status: typing.Optional[EmployeeStatus] = pydantic.Field(default=None)
    """
    Specifies the status of the employees being fetched.
    See [EmployeeStatus](#type-employeestatus) for possible values
    """

    is_owner: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this employee is the owner of the merchant. Each merchant
    has one owner employee, and that employee has full authority over
    the account.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A read-only timestamp in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A read-only timestamp in RFC 3339 format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class InvoiceRecipientTaxIdsParams(typing_extensions.TypedDict):
    """
    Represents the tax IDs for an invoice recipient. The country of the seller account determines
    whether the corresponding `tax_ids` field is available for the customer. For more information,
    see [Invoice recipient tax IDs](https://developer.squareup.com/docs/invoices-api/overview#recipient-tax-ids).
    """

    eu_vat: typing_extensions.NotRequired[str]
    """
    The EU VAT identification number for the invoice recipient. For example, `IE3426675K`.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawGiftCardsClient
from .activities.client import ActivitiesClient
from ..core.request_options import RequestOptions
from ..core.pagination import SyncPager
from ..types.gift_card import GiftCard
from ..types.list_gift_cards_response import ListGiftCardsResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..requests.gift_card import GiftCardParams
from ..types.create_gift_card_response import CreateGiftCardResponse
from ..types.get_gift_card_from_gan_response import GetGiftCardFromGanResponse
from ..types.get_gift_card_from_nonce_response import GetGiftCardFromNonceResponse
from ..types.link_customer_to_gift_card_response import LinkCustomerToGiftCardResponse
from ..types.unlink_customer_from_gift_card_response import UnlinkCustomerFromGiftCardResponse
from ..types.get_gift_card_response import GetGiftCardResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawGiftCardsClient
from .activities.client import AsyncActivitiesClient
from ..core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class GiftCardsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawGiftCardsClient(client_wrapper=client_wrapper)
        self.activities = ActivitiesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawGiftCardsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawGiftCardsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        type: typing.Optional[str] = None,
        state: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        customer_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[GiftCard]:
        """
        Lists all gift cards. You can specify optional filters to retrieve
        a subset of the gift cards. Results are sorted by `created_at` in ascending order.

        Parameters
        ----------
        type : typing.Optional[str]
            If a [type](entity:GiftCardType) is provided, the endpoint returns gift cards of the specified type.
            Otherwise, the endpoint returns gift cards of all types.

        state : typing.Optional[str]
            If a [state](entity:GiftCardStatus) is provided, the endpoint returns the gift cards in the specified state.
            Otherwise, the endpoint returns the gift cards of all states.

        limit : typing.Optional[int]
            If a limit is provided, the endpoint returns only the specified number of results per page.
            The maximum value is 200. The default value is 30.
            For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            If a cursor is not provided, the endpoint returns the first page of the results.
            For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        customer_id : typing.Optional[str]
            If a customer ID is provided, the endpoint returns only the gift cards linked to the specified customer.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[GiftCard]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.gift_cards.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/gift-cards",
            method="GET",
            params={
                "type": type,
                "state": state,
                "limit": limit,
                "cursor": cursor,
                "customer_id": customer_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListGiftCardsResponse,
                    construct_type(
                        type_=ListGiftCardsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    type=type,
                    state=state,
                    limit=limit,
                    cursor=_parsed_next,
                    customer_id=customer_id,
                    request_options=request_options,
                )
                _items = _parsed_response.gift_cards
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        idempotency_key: str,
        location_id: str,
        gift_card: GiftCardParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateGiftCardResponse:
        """
        Creates a digital gift card or registers a physical (plastic) gift card. The resulting gift card
        has a `PENDING` state. To activate a gift card so that it can be redeemed for purchases, call
        [CreateGiftCardActivity](api-endpoint:GiftCardActivities-CreateGiftCardActivity) and create an `ACTIVATE`
        activity with the initial balance. Alternatively, you can use [RefundPayment](api-endpoint:Refunds-RefundPayment)
        to refund a payment to the new gift card.

        Parameters
        ----------
        idempotency_key : str
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        location_id : str
            The ID of the [location](entity:Location) where the gift card should be registered for
            reporting purposes. Gift cards can be redeemed at any of the seller's locations.

        gift_card : GiftCardParams
            The gift card to create. The `type` field is required for this request. The `gan_source`
            and `gan` fields are included as follows:

            To direct Square to generate a 16-digit GAN, omit `gan_source` and `gan`.

            To provide a custom GAN, include `gan_source` and `gan`.
            - For `gan_source`, specify `OTHER`.
            - For `gan`, provide a custom GAN containing 8 to 20 alphanumeric characters. The GAN must be
            unique for the seller and cannot start with the same bank identification number (BIN) as major
            credit cards. Do not use GANs that are easy to guess (such as 12345678) because they greatly
            increase the risk of fraud. It is the responsibility of the developer to ensure the security
            of their custom GANs. For more information, see
            [Custom GANs](https://developer.squareup.com/docs/gift-cards/using-gift-cards-api#custom-gans).

            To register an unused, physical gift card that the seller previously ordered from Square,
            include `gan` and provide the GAN that is printed on the gift card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateGiftCardResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.gift_cards.create(
            idempotency_key="NC9Tm69EjbjtConu",
            location_id="81FN9BNFZTKS4",
            gift_card={"type": "DIGITAL"},
        )
        """
        response = self._raw_client.create(
            idempotency_key=idempotency_key,
            location_id=location_id,
            gift_card=gift_card,
            request_options=request_options,
        )
        return response.data

    def get_from_gan(
        self, *, gan: str, request_options: typing.Optional[RequestOptions] = None
    ) -> GetGiftCardFromGanResponse:
        """
        Retrieves a gift card using the gift card account number (GAN).

        Parameters
        ----------
        gan : str
            The gift card account number (GAN) of the gift card to retrieve.
            The maximum length of a GAN is 255 digits to account for third-party GANs that have been imported.
            Square-issued gift cards have 16-digit GANs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetGiftCardFromGanResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.gift_cards.get_from_gan(
            gan="7783320001001635",
        )
        """
        response = self._raw_client.get_from_gan(gan=gan, request_options=request_options)
        return response.data

    def get_from_nonce(
        self, *, nonce: str, request_options: typing.Optional[RequestOptions] = None
    ) -> GetGiftCardFromNonceResponse:
        """
        Retrieves a gift card using a secure payment token that represents the gift card.

        Parameters
        ----------
        nonce : str
            The payment token of the gift card to retrieve. Payment tokens are generated by the
            Web Payments SDK or In-App Payments SDK.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetGiftCardFromNonceResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.gift_cards.get_from_nonce(
            nonce="cnon:7783322135245171",
        )
        """
        response = self._raw_client.get_from_nonce(nonce=nonce, request_options=request_options)
        return response.data

    def link_customer(
        self, gift_card_id: str, *, customer_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> LinkCustomerToGiftCardResponse:
        """
        Links a customer to a gift card, which is also referred to as adding a card on file.

        Parameters
        ----------
        gift_card_id : str
            The ID of the gift card to be linked.

        customer_id : str
            The ID of the customer to link to the gift card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LinkCustomerToGiftCardResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.gift_cards.link_customer(
            gift_card_id="gift_card_id",
            customer_id="GKY0FZ3V717AH8Q2D821PNT2ZW",
        )
        """
        response = self._raw_client.link_customer(
            gift_card_id, customer_id=customer_id, request_options=request_options
        )
        return response.data

    def unlink_customer(
        self, gift_card_id: str, *, customer_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> UnlinkCustomerFromGiftCardResponse:
        """
        Unlinks a customer from a gift card, which is also referred to as removing a card on file.

        Parameters
        ----------
        gift_card_id : str
            The ID of the gift card to be unlinked.

        customer_id : str
            The ID of the customer to unlink from the gift card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UnlinkCustomerFromGiftCardResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.gift_cards.unlink_customer(
            gift_card_id="gift_card_id",
            customer_id="GKY0FZ3V717AH8Q2D821PNT2ZW",
        )
        """
        response = self._raw_client.unlink_customer(
            gift_card_id, customer_id=customer_id, request_options=request_options
        )
        return response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetGiftCardResponse:
        """
        Retrieves a gift card using the gift card ID.

        Parameters
        ----------
        id : str
            The ID of the gift card to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetGiftCardResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.gift_cards.get(
            id="id",
        )
        """
        response = self._raw_client.get(id, request_options=request_options)
        return response.data


class AsyncGiftCardsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawGiftCardsClient(client_wrapper=client_wrapper)
        self.activities = AsyncActivitiesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawGiftCardsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawGiftCardsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        type: typing.Optional[str] = None,
        state: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        customer_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[GiftCard]:
        """
        Lists all gift cards. You can specify optional filters to retrieve
        a subset of the gift cards. Results are sorted by `created_at` in ascending order.

        Parameters
        ----------
        type : typing.Optional[str]
            If a [type](entity:GiftCardType) is provided, the endpoint returns gift cards of the specified type.
            Otherwise, the endpoint returns gift cards of all types.

        state : typing.Optional[str]
            If a [state](entity:GiftCardStatus) is provided, the endpoint returns the gift cards in the specified state.
            Otherwise, the endpoint returns the gift cards of all states.

        limit : typing.Optional[int]
            If a limit is provided, the endpoint returns only the specified number of results per page.
            The maximum value is 200. The default value is 30.
            For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            If a cursor is not provided, the endpoint returns the first page of the results.
            For more information, see [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination).

        customer_id : typing.Optional[str]
            If a customer ID is provided, the endpoint returns only the gift cards linked to the specified customer.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[GiftCard]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.gift_cards.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/gift-cards",
            method="GET",
            params={
                "type": type,
                "state": state,
                "limit": limit,
                "cursor": cursor,
                "customer_id": customer_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListGiftCardsResponse,
                    construct_type(
                        type_=ListGiftCardsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    type=type,
                    state=state,
                    limit=limit,
                    cursor=_parsed_next,
                    customer_id=customer_id,
                    request_options=request_options,
                )
                _items = _parsed_response.gift_cards
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        idempotency_key: str,
        location_id: str,
        gift_card: GiftCardParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateGiftCardResponse:
        """
        Creates a digital gift card or registers a physical (plastic) gift card. The resulting gift card
        has a `PENDING` state. To activate a gift card so that it can be redeemed for purchases, call
        [CreateGiftCardActivity](api-endpoint:GiftCardActivities-CreateGiftCardActivity) and create an `ACTIVATE`
        activity with the initial balance. Alternatively, you can use [RefundPayment](api-endpoint:Refunds-RefundPayment)
        to refund a payment to the new gift card.

        Parameters
        ----------
        idempotency_key : str
            A unique identifier for this request, used to ensure idempotency. For more information,
            see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        location_id : str
            The ID of the [location](entity:Location) where the gift card should be registered for
            reporting purposes. Gift cards can be redeemed at any of the seller's locations.

        gift_card : GiftCardParams
            The gift card to create. The `type` field is required for this request. The `gan_source`
            and `gan` fields are included as follows:

            To direct Square to generate a 16-digit GAN, omit `gan_source` and `gan`.

            To provide a custom GAN, include `gan_source` and `gan`.
            - For `gan_source`, specify `OTHER`.
            - For `gan`, provide a custom GAN containing 8 to 20 alphanumeric characters. The GAN must be
            unique for the seller and cannot start with the same bank identification number (BIN) as major
            credit cards. Do not use GANs that are easy to guess (such as 12345678) because they greatly
            increase the risk of fraud. It is the responsibility of the developer to ensure the security
            of their custom GANs. For more information, see
            [Custom GANs](https://developer.squareup.com/docs/gift-cards/using-gift-cards-api#custom-gans).

            To register an unused, physical gift card that the seller previously ordered from Square,
            include `gan` and provide the GAN that is printed on the gift card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateGiftCardResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.gift_cards.create(
                idempotency_key="NC9Tm69EjbjtConu",
                location_id="81FN9BNFZTKS4",
                gift_card={"type": "DIGITAL"},
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            idempotency_key=idempotency_key,
            location_id=location_id,
            gift_card=gift_card,
            request_options=request_options,
        )
        return response.data

    async def get_from_gan(
        self, *, gan: str, request_options: typing.Optional[RequestOptions] = None
    ) -> GetGiftCardFromGanResponse:
        """
        Retrieves a gift card using the gift card account number (GAN).

        Parameters
        ----------
        gan : str
            The gift card account number (GAN) of the gift card to retrieve.
            The maximum length of a GAN is 255 digits to account for third-party GANs that have been imported.
            Square-issued gift cards have 16-digit GANs.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetGiftCardFromGanResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.gift_cards.get_from_gan(
                gan="7783320001001635",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get_from_gan(gan=gan, request_options=request_options)
        return response.data

    async def get_from_nonce(
        self, *, nonce: str, request_options: typing.Optional[RequestOptions] = None
    ) -> GetGiftCardFromNonceResponse:
        """
        Retrieves a gift card using a secure payment token that represents the gift card.

        Parameters
        ----------
        nonce : str
            The payment token of the gift card to retrieve. Payment tokens are generated by the
            Web Payments SDK or In-App Payments SDK.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetGiftCardFromNonceResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.gift_cards.get_from_nonce(
                nonce="cnon:7783322135245171",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get_from_nonce(nonce=nonce, request_options=request_options)
        return response.data

    async def link_customer(
        self, gift_card_id: str, *, customer_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> LinkCustomerToGiftCardResponse:
        """
        Links a customer to a gift card, which is also referred to as adding a card on file.

        Parameters
        ----------
        gift_card_id : str
            The ID of the gift card to be linked.

        customer_id : str
            The ID of the customer to link to the gift card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LinkCustomerToGiftCardResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.gift_cards.link_customer(
                gift_card_id="gift_card_id",
                customer_id="GKY0FZ3V717AH8Q2D821PNT2ZW",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.link_customer(
            gift_card_id, customer_id=customer_id, request_options=request_options
        )
        return response.data

    async def unlink_customer(
        self, gift_card_id: str, *, customer_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> UnlinkCustomerFromGiftCardResponse:
        """
        Unlinks a customer from a gift card, which is also referred to as removing a card on file.

        Parameters
        ----------
        gift_card_id : str
            The ID of the gift card to be unlinked.

        customer_id : str
            The ID of the customer to unlink from the gift card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UnlinkCustomerFromGiftCardResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.gift_cards.unlink_customer(
                gift_card_id="gift_card_id",
                customer_id="GKY0FZ3V717AH8Q2D821PNT2ZW",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.unlink_customer(
            gift_card_id, customer_id=customer_id, request_options=request_options
        )
        return response.data

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetGiftCardResponse:
        """
        Retrieves a gift card using the gift card ID.

        Parameters
        ----------
        id : str
            The ID of the gift card to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetGiftCardResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.gift_cards.get(
                id="id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class DeleteInvoiceAttachmentResponseParams(typing_extensions.TypedDict):
    """
    Represents a [DeleteInvoiceAttachment](api-endpoint:Invoices-DeleteInvoiceAttachment) response.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .terminal_checkout import TerminalCheckoutParams


class SearchTerminalCheckoutsResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information about errors encountered during the request.
    """

    checkouts: typing_extensions.NotRequired[typing.Sequence[TerminalCheckoutParams]]
    """
    The requested search result of `TerminalCheckout` objects.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor to be used in a subsequent request. If empty,
    this is the final response.
    
    See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more information.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .country import Country
from .currency import Currency
from .merchant_status import MerchantStatus
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Merchant(UncheckedBaseModel):
    """
    Represents a business that sells with Square.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Square-issued ID of the merchant.
    """

    business_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the merchant's overall business.
    """

    country: Country = pydantic.Field()
    """
    The country code associated with the merchant, in the two-letter format of ISO 3166. For example, `US` or `JP`.
    See [Country](#type-country) for possible values
    """

    language_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The code indicating the [language preferences](https://developer.squareup.com/docs/build-basics/general-considerations/language-preferences) of the merchant, in [BCP 47 format](https://tools.ietf.org/html/bcp47#appendix-A). For example, `en-US` or `fr-CA`.
    """

    currency: typing.Optional[Currency] = pydantic.Field(default=None)
    """
    The currency associated with the merchant, in ISO 4217 format. For example, the currency code for US dollars is `USD`.
    See [Currency](#type-currency) for possible values
    """

    status: typing.Optional[MerchantStatus] = pydantic.Field(default=None)
    """
    The merchant's status.
    See [MerchantStatus](#type-merchantstatus) for possible values
    """

    main_location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the [main `Location`](https://developer.squareup.com/docs/locations-api#about-the-main-location) for this merchant.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The time when the merchant was created, in RFC 3339 format.
       For more information, see [Working with Dates](https://developer.squareup.com/docs/build-basics/working-with-dates).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class SquareAccountDetailsParams(typing_extensions.TypedDict):
    """
    Additional details about Square Account payments.
    """

    payment_source_token: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Unique identifier for the payment source used for this payment.
    """

    errors: typing_extensions.NotRequired[typing.Optional[typing.Sequence[ErrorParams]]]
    """
    Information about errors encountered during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .loyalty_program import LoyaltyProgramParams


class ListLoyaltyProgramsResponseParams(typing_extensions.TypedDict):
    """
    A response that contains all loyalty programs.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    programs: typing_extensions.NotRequired[typing.Sequence[LoyaltyProgramParams]]
    """
    A list of `LoyaltyProgram` for the merchant.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

V1OrderHistoryEntryAction = typing.Union[
    typing.Literal["ORDER_PLACED", "DECLINED", "PAYMENT_RECEIVED", "CANCELED", "COMPLETED", "REFUNDED", "EXPIRED"],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams


class OrderLineItemAppliedServiceChargeParams(typing_extensions.TypedDict):
    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID that identifies the applied service charge only within this order.
    """

    service_charge_uid: str
    """
    The `uid` of the service charge that the applied service charge represents. It must
    reference a service charge present in the `order.service_charges` field.
    
    This field is immutable. To change which service charges apply to a line item,
    delete and add a new `OrderLineItemAppliedServiceCharge`.
    """

    applied_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money applied by the service charge to the line item.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

TenderType = typing.Union[
    typing.Literal[
        "CARD",
        "CASH",
        "THIRD_PARTY_CARD",
        "SQUARE_GIFT_CARD",
        "NO_SALE",
        "BANK_ACCOUNT",
        "WALLET",
        "BUY_NOW_PAY_LATER",
        "SQUARE_ACCOUNT",
        "OTHER",
    ],
    typing.Any,
]
